----------------------------------------------------------------------------------
-- Company: Digilent RO
-- Engineer: Mircea Dabacan
-- 
-- Create Date:    11:58:05 02/28/2008 
-- Design Name: 
-- Module Name:    MullAcc - Behavioral 
-- Project Name:   ActivePowerMeter
-- Target Devices: 
-- Tool versions: 
-- Description:    Multiply accumulate component:
--      The component gets simultaneous samples of two analog signals:
--          - vecI = input current
--          - vecU = input voltage  
--      Each couple of vecI and vecU are multiplied to get 
--          - vecPinst = instant Power
--      2**k vecInstP samples are added resulting
--          - vecE = the active energy over the measurement period.
--      vecE is divided by the number of samples, to get 
--          - vecP = the active power.
--      Choosing a power of 2 as umber of samples, transforms the above division 
--      into shifting.
--
--      All vecI, vecU, vecE, vecPinst, vecP are signed values. 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_SIGNED.ALL;

entity MullAcc is
  Port ( ck : in  STD_LOGIC; -- Multiply accumulate clock
        -- a sample couple (vecI and vecU) is read each rising edge of ck signal
    vecI : in STD_LOGIC_VECTOR (11 downto 0);  -- input current samples 
    vecU : in STD_LOGIC_VECTOR (11 downto 0);  -- input voltage samples 
    vecP : out STD_LOGIC_VECTOR (11 downto 0); -- Active Power 
	                                            -- over the measurement period
    vecPinst : out STD_LOGIC_VECTOR (11 downto 0)); -- Instant Power 
	                                           
end MullAcc;

architecture Behavioral of MullAcc is
  constant k: integer :=16;
    -- number of samples added for a measurement = 2**k
    -- measurement period = 2**k*(ck period)
  signal cntSamples: integer range 0 to 2**k-1 := 0;
  signal vecE: std_logic_vector(2*12+k-1 downto 0):=(others => '0');
    -- accumulates the sample products; 
	 -- expresses the active energy over the meaurement period
  
begin

  process(ck)
    variable vecEvar: std_logic_vector(2*12+k-1 downto 0);
    variable vecPinstVar: std_logic_vector(2*12-1 downto 0);
  begin

  vecPinstVar := vecI * vecU;
  vecEvar := vecE + vecPinstVar;

  vecPinst <= vecPinstVar(2*12-1 downto 12);

	 if ck'event and ck = '1' then
	   if cntSamples = 2**k-1 then
		  cntSamples <= 0;
		  vecE <= (others => '0');
		  vecP <= vecEvar(2*12+k-1 downto 12+k);
		else
		  cntSamples <= cntSamples + 1;
		  vecE <= vecEvar;
		end if;
    end if;
  end process;	 

end Behavioral;

