----------------------------------------------------------------------------------
-- Company: Digilent RO
-- Engineer: Mircea Dabacan
-- 
-- Create Date:    19:03:35 02/28/2008 
-- Design Name: 
-- Module Name:    PowerMeter - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--    The module implements a power meter. Two input signals are acquired,
--    representing a current and a voltage.
--    External circuitry is required to:
--      - Convert the current I to a voltage signal VI in range 0 to 3.3V as follows:
--        - when current I = 0mA => VI = 1.65V;
--        - when current I = MaxIValue (positive) => VI = 3.3V;
--        - when current I = -MaxIValue (negative) => VI = 0V;
--      - Scale the input voltage V to a voltage VU in range: 0 to 3.3V as follows: 
--        - when voltage U = 0mV => VU = 1.65V;
--        - when voltage U = MaxUValue (positive) => VU = 3.3V;
--        - when voltage U = -MaxUValue (negative) => VU = 0V;
--      - connect VI and VU to the voltage inputs of Pmod AD1.
--      - connect the Pmod AD1 to the used Digilent system board.

--    The module instantiates an AD1 component for acquiring unsigned samples of:
--      - vecIUnsigned - digital image of VI
--      - vecUUnsigned - digital image of VU
--
--    The input vectors above are converted to signed vectors:
--      - vecISigned - digital image of I, normalized to range (-MaxIValue;+MaxIValue)
--      - vecUSigned - digital image of U, normalized to range (-MaxUValue;+MaxUValue)
--
--    The signed values above are supplied to MullAcc component, which returns:
--      - vecPinstSigned - digital image of the instant power Pinst, 
--          normalized to range (-MaxPValue;+MaxPValue)    
--      - vecPSigned - digital image of the mean effective power over measurement period P, 
--          normalized to range (-MaxPValue;+MaxPValue)    
--    In the ranges above, MaxPValue = MaxIValue*MaxUValue.
--
--    vecPinstSigned is converted to an unsigned value vecPinstUnsigned, which is sent to 
--    a DA1 component, which sends it further to a Pmod DA1. 
--    The Pmod AD1 output signal VP, represents the instant power, shifted by 1.65V:
--      - when Pinst = 0 => VP = 1.65V
--      - when Pinst = -MaxPValue => VP = 0V
--      - when Pinst = MaxPValue => VP = 3.3V
--
--    vecPSigned is converted to a "Sign and Magnitude" BCD representation: 
--      - bitPSign - the sign (0 for +, 1 for -), 
--      - vecPMagnitudeBcd - the P absolute value
-- 
--    The signals above are sent to a seven segment display.
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity PowerMeter is
  Port (ck : in  STD_LOGIC;  -- System clock (50MHz)
        bitSclkAdc, bitCsAdc: out std_logic;  -- Pmod AD1 pins
        bitD1Adc, bitD2Adc: in std_logic;  -- Pmod AD1 pins
        bitClkDac, bitnSyncDac, bitD1DAC, bitD2DAC: out std_logic;  -- Pmod DA1 pins
        seg: out std_logic_vector(6 downto 0); -- segment outputs
        an : out std_logic_vector(3 downto 0));-- anode select signals
end PowerMeter;

architecture Behavioral of PowerMeter is

  signal ckSample : STD_LOGIC;
  signal cnt1 : integer range 0 to 763; -- divider for ckSample

  signal vecIUnsigned : STD_LOGIC_VECTOR (11 downto 0);
  signal vecUUnsigned : STD_LOGIC_VECTOR (11 downto 0);
  signal vecPinstUnsigned : STD_LOGIC_VECTOR (11 downto 0);
  signal vecPUnsigned : STD_LOGIC_VECTOR (11 downto 0);
  signal vecISigned : STD_LOGIC_VECTOR (11 downto 0);
  signal vecUSigned : STD_LOGIC_VECTOR (11 downto 0);
  signal vecPinstSigned : STD_LOGIC_VECTOR (11 downto 0);
  signal vecPSigned : STD_LOGIC_VECTOR (11 downto 0);

  signal HEX : std_logic_vector(3 downto 0);  -- a digit for 7 seg display
  signal cnt : std_logic_vector(18 downto 0):="0000000000000000000";-- divider for 7SD

  signal bitPSign : std_logic;  -- sign bit for active Power;'0' => positive, '1' => negative
  signal vecPMagnitude : std_logic_vector(10 downto 0);  -- Power magnitude
  signal vecPMagnitudeM : std_logic_vector(20 downto 0);  -- Power magnitude multiplied by 1000
  signal vecPMagnitudeS : std_logic_vector(11 downto 0);  -- Power magnitude scaled to 1000
  signal vecPMagnitudeBcd : std_logic_vector(11 downto 0);  -- Power magnitude BCD
						 
 	COMPONENT AD1_controller
	PORT(
		CLK : IN std_logic;
		RST : IN std_logic;
		SDATA1 : IN std_logic;
		SDATA2 : IN std_logic;
		START : IN std_logic;          
		SCLK : OUT std_logic;
		CS : OUT std_logic;
		DATA1 : OUT std_logic_vector(11 downto 0);
		DATA2 : OUT std_logic_vector(11 downto 0);
		DONE : OUT std_logic
		);
	END COMPONENT;

	COMPONENT MullAcc
	PORT(
		ck : IN std_logic;
		vecI : IN std_logic_vector(11 downto 0);
		vecU : IN std_logic_vector(11 downto 0);          
		vecP : OUT std_logic_vector(11 downto 0);
		vecPinst : OUT std_logic_vector(11 downto 0)
		);
	END COMPONENT;

	COMPONENT DA2_controller
	PORT(
		CLK : IN std_logic;
		RST : IN std_logic;
		DATA1 : IN std_logic_vector(11 downto 0);
		DATA2 : IN std_logic_vector(11 downto 0);
		START : IN std_logic;          
		D1 : OUT std_logic;
		D2 : OUT std_logic;
		CLK_OUT : OUT std_logic;
		nSYNC : OUT std_logic;
		DONE : OUT std_logic
		);
	END COMPONENT;

	COMPONENT BinToBcdEncoder_nDigits
	PORT(
		vecIn : IN std_logic_vector(11 downto 0);          
		bcdOut : OUT std_logic_vector(11 downto 0)
		);
	END COMPONENT;

begin

  -- generating a clock signal of frequency ~2**16Hz
  SampleClock: process (ck)
   begin
     if rising_edge(ck) then  -- dividing 50MHz by 763
       if cnt1 = 763-1 then
	      cnt1 <= 0;
       else 
         cnt1 <=cnt1 + 1;
       end if; 
    end if;

    if rising_edge(ck) then  -- about 50% duty cycle
       if cnt1 = 763-1 then
	      ckSample <= '0';
       elsif cnt1 = 381 then 
         ckSample <= '1';
       end if; 
    end if;

  end process;

	Inst_AD1_controller: AD1_controller PORT MAP(
		CLK => ck,
		RST => '0',
		SDATA1 => bitD1Adc,
		SDATA2 => bitD2Adc,
		SCLK => bitSclkAdc,
		CS => bitCsAdc,
		DATA1 => vecIUnsigned,
		DATA2 => vecUUnsigned,
		START => ckSample,
		DONE => open
	);

  -- converting Unsigned to Signed
  vecISigned <= (not vecIUnsigned(11)) & vecIUnsigned(10 downto 0);
  vecUSigned <= (not vecUUnsigned(11)) & vecUUnsigned(10 downto 0);

  Inst_MullAcc: MullAcc PORT MAP(
    ck => ckSample,
    vecI => vecISigned,
    vecU => vecUSigned,
    vecP => vecPSigned,
    vecPinst => vecPinstSigned
    );

  -- converting Signed to Unsigned
  vecPUnsigned <= (not vecPSigned(11)) & vecPSigned(10 downto 0);
  vecPinstUnsigned <= (not vecPinstSigned(11)) & vecPinstSigned(10 downto 0);
 
	Inst_DA2_controller: DA2_controller PORT MAP(
		CLK => ck,
		RST => '0',
		D1 => bitD1DAC,
		D2 => bitD2DAC,
		CLK_OUT => bitClkDac,
		nSYNC => bitnSyncDac,
		DATA1 => vecPUnsigned,
		DATA2 => vecPinstUnsigned,
		START => ckSample,
		DONE => open
	);

  -- converting Signed to "Sign and Magnitude"
  bitPSign <= vecPSigned(11);  --'0' => positive, '1' => negative
  vecPMagnitude <= vecPSigned(10 downto 0) when bitPSign = '0' else
                   (not vecPSigned(10 downto 0)) + '1';
						 
  -- scaling to the range 0...1000
  vecPMagnitudeM <= vecPMagnitude * conv_std_logic_vector(1000,10); -- *1000
  vecPMagnitudeS <= '0' & vecPMagnitudeM(20 downto 10);             -- /2048
            -- leading '0' added to complete to 12 bit for 
  
  -- converting to BCD
	Inst_BinToBcdEncoder_nDigits: BinToBcdEncoder_nDigits PORT MAP(
		vecIn => vecPMagnitudeS,
		bcdOut => vecPMagnitudeBcd
	);
  
  -- displaying the BCD value 
div: process (ck)
 begin
  if rising_edge(ck) then
   cnt <=cnt + "0000000000000000001";
  end if;
 end process;

with cnt(18 downto 17) select
 hex <= vecPMagnitudeBcd(3 downto 0) when "00",
        vecPMagnitudeBcd(7 downto 4) when "01",
        vecPMagnitudeBcd(11 downto 8) when "10",
       "111" & bitPSign when others; -- "1110" for positive, "1111" for negative

with cnt(18 downto 17) select
 an <= "1110" when "00",
       "1101" when "01",
       "1011" when "10",
       "0111" when others; 

--HEX-to-seven-segment decoder
-- 
-- segment encoding
--      0
--     ---  
--  5 |   | 1
--     ---   <- 6
--  4 |   | 2
--     ---
--      3
   
    with HEX select
   seg<= "1000000" when "0000",   --0
         "1111001" when "0001",   --1
         "0100100" when "0010",   --2
         "0110000" when "0011",   --3
         "0011001" when "0100",   --4
         "0010010" when "0101",   --5
         "0000010" when "0110",   --6
         "1111000" when "0111",   --7
         "0000000" when "1000",   --8
         "0010000" when "1001",   --9
         "0111111" when "1111",   -- minus sign
         "1111111" when others;   -- nothing, for plus sign
 
end Behavioral;

